VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsResizer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

    '
    '   Type or resizing
    '
    Public Enum eResizerRules
        SizeTo_Width        '   Stretch width to fill container
        SizeTo_height       '   Stretch height to fill container
        SizeTo_Right        '   Stretch right to fill container
        SizeTo_Bottom       '   Stretch down to fill container
        Align_BottomToBottom '   Align to bottom of container
        Align_LeftToLeft    '   Align to left of container
        Align_RightToRight  '   Align to right side of container
        Align_TopToTop      '   Align to top of container
        Uneven_Horizontal   '   Stretch unevenly horizontal
        unEven_Vertical     '   Stretch Unevenly vertical
        Even_Horizontal     '   Stretch equally horizontal
        Even_Vertical       '   Stretch equally vertical
        SizeTo_Source       '   Stretch to fill container
    End Enum
    
    '
    '   A sizing rule
    '
    Private Type RULE_INSTANCE
        Rule    As eResizerRules
        Source  As Object
        Dest()  As Variant
        Border  As Long
    End Type

    '
    '   Collection of rules
    '
    Private maAllInst()             As RULE_INSTANCE

    '
    '   Rectangle
    '
    Private Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
    End Type
    
    '
    '   Pointer to form
    '
    Private WithEvents mobjForm     As Form
Attribute mobjForm.VB_VarHelpID = -1
    
Public Sub Resize()

    '
    '   Pass the resize event
    '
    Call mobjForm_Resize
    
End Sub
Public Property Set TargetForm(Frm As Form)
    Set mobjForm = Frm
End Property
Public Sub AddRule(Rule As eResizerRules, Source As Object, Border As Long, ParamArray Dest() As Variant)

    Dim lRules As Long
    
    '
    '   Get the current number of rules defined
    '
    lRules = UBound(maAllInst)
    
    '
    '   Redimension the rules array
    '
    If lRules = 0 Then
        ReDim maAllInst(1 To 1)
    Else
        ReDim Preserve maAllInst(1 To lRules + 1)
    End If
    
    '
    '   Save the new rull
    '
    With maAllInst(lRules + 1)
        .Rule = Rule
        Set .Source = Source
        .Dest = Dest
        .Border = Border
    End With
    
End Sub
Private Sub Class_Initialize()

    '
    '   Ensure we have an initialized array
    '
    ReDim maAllInst(0)
    
End Sub
Private Sub mobjForm_Resize()
    
    Dim lRule As Long
    
    '
    '   Loop through all Rules and Apply Them
    '
    For lRule = LBound(maAllInst) To UBound(maAllInst)
        Call mApplyRule(maAllInst(lRule))
    Next lRule
    
End Sub
Private Sub mApplyRule(Rule As RULE_INSTANCE)

    On Error Resume Next
    
    Dim udtSrcExtents   As RECT
    
    Dim lBorder As Long
    Dim lBottomNext As Long
    Dim lControl As Long
    Dim lHeightRemaining  As Long
    Dim lLeftAnchor As Long
    Dim lLeftNext   As Long
    Dim lLowerBound As Long
    Dim lRightNext As Long
    Dim lTopAnchor  As Long
    Dim lTopNext As Long
    Dim lUpperBound As Long
    Dim lWidthRemaining  As Long
    
    '
    '   Determine Bounding Rectangle
    '
    With Rule
    
        '
        '   We need to treat forms different than other containers
        '
        If TypeOf .Source Is Form Then
            udtSrcExtents.Left = 0
            udtSrcExtents.Top = 0
            udtSrcExtents.Right = .Source.ScaleWidth
            udtSrcExtents.Bottom = .Source.ScaleHeight
        Else
            udtSrcExtents.Left = .Source.Left
            udtSrcExtents.Top = .Source.Top
'            udtSrcExtents.Left = 0
'            udtSrcExtents.Top = 0
            udtSrcExtents.Right = .Source.Left + .Source.Width
            udtSrcExtents.Bottom = .Source.Top + .Source.Height
        End If
        
        '
        '   Get the border
        '
        lBorder = .Border
        
        '
        '   Apply rule to all controls in Dest array
        '
        lLowerBound = LBound(.Dest)
        lUpperBound = UBound(.Dest)
        For lControl = lLowerBound To lUpperBound
            With .Dest(lControl)
                '
                '   Determine rule type
                '
                Select Case Rule.Rule
                    '
                    '   Fill width
                    '
                    Case eResizerRules.SizeTo_Width
                        '
                        '   Make the control the same width as the container and left align it
                        '
                        .Left = lBorder
                        .Width = udtSrcExtents.Right - udtSrcExtents.Left - lBorder * 2 - udtSrcExtents.Left
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Fill height
                    '
                    Case eResizerRules.SizeTo_height
                        '
                        '   Make the control the same height as the container and top align it
                        '
                        .Top = lBorder
                        .Height = udtSrcExtents.Bottom - udtSrcExtents.Top - lBorder * 2 - udtSrcExtents.Top
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Stretch right
                    '
                    Case eResizerRules.SizeTo_Right
                        '
                        '   Widen the control to fill the right side of the container
                        '
                        .Width = udtSrcExtents.Right - .Left - lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Stretch down
                    '
                    Case eResizerRules.SizeTo_Bottom
                        '
                        '   Lengthen the control to fill the bottom part of the container
                        '
                        .Height = udtSrcExtents.Bottom - .Top - lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Align to right edge of containter
                    '
                    Case eResizerRules.Align_BottomToBottom
                        '
                        '   Align the control with the bottom of the container
                        '
                        .Top = udtSrcExtents.Bottom - .Height - lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Align to left edge of containter
                    '
                    Case eResizerRules.Align_LeftToLeft
                        '
                        '   Align the control with the left side of the container
                        '
                        .Left = udtSrcExtents.Left + lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Align to right edge of containter
                    '
                    Case eResizerRules.Align_RightToRight
                        '
                        '   Align the control with the right side of the container
                        '
                        .Left = udtSrcExtents.Right - .Width - lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Align to top edge of containter
                    '
                    Case eResizerRules.Align_TopToTop
                        '
                        '   Align the control with the top of the container
                        '
                        .Top = udtSrcExtents.Top + lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   Make the controls the same width and fill width of container
                    '
                    Case eResizerRules.Even_Horizontal
                        '
                        '   Is this the first control in the list?
                        '
                        If (lLowerBound = lControl) Then
                            '
                            '   Yes -- Align it with the left side of the container
                            '
                            lLeftNext = lBorder
                        End If
                        '
                        '   Position and stretch the control
                        '
                        .Left = lLeftNext
                        .Width = (udtSrcExtents.Right - udtSrcExtents.Left - lBorder * (lUpperBound - lLowerBound + 2) - udtSrcExtents.Left) / (lUpperBound - lLowerBound + 1)
                        '
                        '   Point to the left edge of the next control
                        '
                        lLeftNext = .Left + .Width + lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   The first control is positioned one border below the top of the container
                    '   Make all the controls the same height and align them top to one border below the bottom of the one above
                    '
                    Case eResizerRules.Even_Vertical
                        '
                        '   Is this the first control in the list?
                        '
                        If (lLowerBound = lControl) Then
                            '
                            '   Yes -- Align it with the top side of the container
                            '
                            lTopNext = lBorder
                        End If
                        '
                        '   Position and stretch the control
                        '
                        .Top = lTopNext
                        'Debug.Print "Calculated Height " & (udtSrcExtents.Bottom - udtSrcExtents.Top - lBorder * (lUpperBound - lLowerBound + 2) - udtSrcExtents.Top) / (lUpperBound - lLowerBound + 1)
                        .Height = (udtSrcExtents.Bottom - udtSrcExtents.Top - lBorder * (lUpperBound - lLowerBound + 2) - udtSrcExtents.Top) / (lUpperBound - lLowerBound + 1)
                        'Debug.Print "Stored Height " & Rule.Dest(lControl).Height
                        '
                        '   Point to the left edge of the next control
                        '
                        lTopNext = .Top + .Height + lBorder
                        'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                    '
                    '   The first control on the list is the one that is resized.
                    '   The other controls are moved left and right next to each other.
                    '   Finally, the first control is resized horizontally to fill the remaining space
                    '
                    Case eResizerRules.Uneven_Horizontal
                        If (lLowerBound = lControl) Then
                            '
                            '   Save the anchor point (determines if the other controls are to the left or right
                            '
                            lLeftAnchor = .Left
                            '
                            '   Set the width remaining to the entire interior of the container less the borders
                            '
                            lWidthRemaining = udtSrcExtents.Right - udtSrcExtents.Left - lBorder * (lUpperBound - lLowerBound + 2)
                            'Debug.Print "Start Uneven Vertical " & lWidthRemaining
                            '
                            '   Initialize the right alignment point
                            '
                            lRightNext = udtSrcExtents.Right - udtSrcExtents.Left - lBorder - udtSrcExtents.Left
                            '
                            '   Initialize the left alignment point
                            '
                            lLeftNext = lBorder
                        '
                        '   If we are processing the last control we need to resize the first control
                        '
                        ElseIf (lUpperBound = lControl) Then
                            '
                            '   But first we process the current control
                            '   We need to determine if it is left or right of the anchor point
                            '
                            If (.Left > lLeftAnchor) Then
                                '
                                '   It is to the right of the first control
                                '
                                .Left = lRightNext - .Width
                                lRightNext = .Left - lBorder
                            Else
                                '
                                '   It is to the left of the first control
                                '
                                .Left = lLeftNext
                                lLeftNext = .Left + .Width + lBorder
                            End If
                            lWidthRemaining = lWidthRemaining - .Width
                            If (0 > lWidthRemaining) Then
                                lWidthRemaining = 0
                            End If
                            'Debug.Print Rule.Dest(lUpperBound).Name & "|" & Rule.Dest(lUpperBound).Top & "|" & Rule.Dest(lUpperBound).Left & "|" & Rule.Dest(lUpperBound).Height & "|" & Rule.Dest(lUpperBound).Width
                            'Debug.Print "Width remaining " & lWidthRemaining
                            '
                            '   Now position the first control to fill the space
                            '
                            Rule.Dest(lLowerBound).Left = lLeftNext
                            Rule.Dest(lLowerBound).Width = lWidthRemaining
                            'Debug.Print Rule.Dest(lLowerBound).Name & "|" & Rule.Dest(lLowerBound).Top & "|" & Rule.Dest(lLowerBound).Left & "|" & Rule.Dest(lLowerBound).Height & "|" & Rule.Dest(lLowerBound).Width
                        '
                        '   The control is neither first nor last
                        '
                        Else
                            '
                            '   We need to determine if it is left or right of the anchor points
                            '
                            If (.Left > lLeftAnchor) Then
                                '
                                '   It is to the right -- position it
                                '
                                .Left = lRightNext - .Width
                                lRightNext = .Left - lBorder
                            Else
                                '
                                '   It is to the left -- position it
                                '
                                .Left = lLeftNext
                                lLeftNext = .Left + .Width + lBorder
                            End If
                            '
                            '   Adjust the remaining width
                            '
                            lWidthRemaining = lWidthRemaining - .Width
                            If (0 > lWidthRemaining) Then
                                lWidthRemaining = 0
                            End If
                            'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                            'Debug.Print "Width remaining " & lWidthRemaining
                        End If
                    '
                    '   The first control on the list is the one that is resized.
                    '   The other controls are moved up and down next to each other.
                    '   Finally, the first control is resized vertically to fill the remaining space
                    '
                    Case eResizerRules.unEven_Vertical
                        If (lLowerBound = lControl) Then
                            '
                            '   Save the anchor point (determines if the other controls are to the top or bottom
                            '
                            lTopAnchor = .Top
                            '
                            '   Set the height remaining to the entire interior of the container less the borders
                            '
                            lHeightRemaining = udtSrcExtents.Bottom - udtSrcExtents.Top - lBorder * (lUpperBound - lLowerBound + 2)
                            'Debug.Print "Start Uneven Vertical " & lHeightRemaining
                            '
                            '   Initialize the bottom alignment point
                            '
                            lBottomNext = udtSrcExtents.Bottom - udtSrcExtents.Top - lBorder - udtSrcExtents.Top
                            '
                            '   Initialize the top alignment point
                            '
                            lTopNext = lBorder
                        '
                        '   If we are processing the last control we need to resize the first control
                        '
                        ElseIf (lUpperBound = lControl) Then
                            '
                            '   But first we process the current control
                            '   We need to determine if it is top or bottom of the anchor point
                            '
                            If (.Top > lTopAnchor) Then
                                '
                                '   It is above the first control
                                '
                                .Top = lBottomNext - .Height
                                lBottomNext = .Top - lBorder
                            Else
                                '
                                '   It is below the first control
                                '
                                .Top = lTopNext
                                lTopNext = .Top + .Height + lBorder
                            End If
                            lHeightRemaining = lHeightRemaining - .Height
                            If (0 > lHeightRemaining) Then
                                lHeightRemaining = 0
                            End If
                            'Debug.Print Rule.Dest(lUpperBound).Name & "|" & Rule.Dest(lUpperBound).Top & "|" & Rule.Dest(lUpperBound).Left & "|" & Rule.Dest(lUpperBound).Height & "|" & Rule.Dest(lLowerBound).Width
                            'Debug.Print "Height remaining " & lHeightRemaining
                            '
                            '   Now position the first control to fill the space
                            '
                            Rule.Dest(lLowerBound).Top = lTopNext
                            Rule.Dest(lLowerBound).Height = lHeightRemaining
                            'Debug.Print Rule.Dest(lLowerBound).Name & "|" & Rule.Dest(lLowerBound).Top & "|" & Rule.Dest(lLowerBound).Left & "|" & Rule.Dest(lLowerBound).Height & "|" & Rule.Dest(lLowerBound).Width
                        '
                        '   The control is neither first nor last
                        '
                        Else
                            '
                            '   We need to determine if it is above or below the anchor point
                            '
                            If (.Top > lTopAnchor) Then
                                '
                                '   It is above the first control
                                '
                                .Top = lBottomNext - .Height
                                lBottomNext = .Top - lBorder
                            Else
                                '
                                '   It is below the first control
                                '
                                .Top = lTopNext
                                lTopNext = .Top + .Height + lBorder
                            End If
                            '
                            '   Adjust the remaing height
                            '
                            lHeightRemaining = lHeightRemaining - .Height
                            If (0 > lHeightRemaining) Then
                                lHeightRemaining = 0
                            End If
                            'Debug.Print Rule.Dest(lControl).Name & "|" & Rule.Dest(lControl).Top & "|" & Rule.Dest(lControl).Left & "|" & Rule.Dest(lControl).Height & "|" & Rule.Dest(lControl).Width
                            'Debug.Print "Height remaining " & lHeightRemaining
                        End If
                    '
                    '   Fill the container with the one control
                    '
                    Case SizeTo_Source
                        .Top = udtSrcExtents.Top
                        .Height = udtSrcExtents.Bottom - udtSrcExtents.Top
                        .Left = udtSrcExtents.Left
                        .Width = udtSrcExtents.Right - udtSrcExtents.Left
                End Select
            End With
        Next lControl
    End With
    'Debug.Print ""
End Sub
